/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.cache.aop;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.PropertyConfigurator;
import org.jboss.cache.aop.test.Person;
import org.jboss.cache.aop.test.Resource;

import javax.naming.Context;
import java.util.Properties;

/**
 *
 * @author Ben Wang
 */

public class ReplicatedByteAopTest extends TestCase
{
   Log log_= LogFactory.getLog(ReplicatedByteAopTest.class);
   PojoCache cache_;
   PojoCache cache1_;

   public ReplicatedByteAopTest(String name)
   {
      super(name);
   }

   protected void setUp() throws Exception
   {
      super.setUp();
      Properties prop = new Properties();
      prop.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.cache.transaction.DummyContextFactory");
      cache_ = new PojoCache();
      PropertyConfigurator config = new PropertyConfigurator(); // configure tree cache.
      config.configure(cache_, "META-INF/replSync-service.xml");

      cache1_ = new PojoCache();
      config.configure(cache1_, "META-INF/replSync-service.xml");
      cache_.start();
      cache1_.start();
   }

   protected void tearDown() throws Exception
   {
      super.tearDown();
      cache_.stop();
      cache1_.stop();
   }

   public void testSimple() throws Exception
   {
      log_.info("testSimple() ....");

      Resource res = new Resource();
      res.setName("mapping");
      res.setConnection("wire");
      String s = "This is a test";
      byte by = 1;
      byte[] b = new byte[1];
      b[0] = by;
      res.setByte(b);

      cache_.putObject("/resource", res);

      Resource remote = (Resource)cache1_.getObject("/resource");
      assertEquals("Name ", res.getName(), remote.getName());
      assertEquals("Bytes ", res.getByte()[0], remote.getByte()[0]);
   }

   public static Test suite() throws Exception
   {
      return new TestSuite(ReplicatedByteAopTest.class);
   }


   public static void main(String[] args) throws Exception
   {
      junit.textui.TestRunner.run(suite());
   }

}
