/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.cache.multiplexer;

import org.jgroups.Channel;
import org.jgroups.JChannelFactory;
import org.w3c.dom.Element;

/**
 * Delegates to a JChannelFactory but exposes a StandardMBean interface.
 * JChannelFactory does not expose a StandardMBean interface, making it
 * fairly heavy to work with.  This class just exposes the subset of
 * JChannelFactory methods needed by JBC or by the test suite.
 * 
 * @author <a href="brian.stansberry@jboss.com">Brian Stansberry</a>
 * @version $Revision: 1.1.2.2 $
 */
public class MuxChannelFactory implements MuxChannelFactoryMBean
{
   private final JChannelFactory delegate;
   private final Element muxConfig;
   
   public MuxChannelFactory(JChannelFactory jChannelFactory, Element muxConfig) throws Exception
   {
      this.delegate = jChannelFactory;
      delegate.setMultiplexerConfig(muxConfig);
      this.muxConfig = muxConfig;
   }
   
   public Channel createMultiplexerChannel(String stack_name, String id) throws Exception
   {
      return delegate.createMultiplexerChannel(stack_name, id);
   }  
   
   public Element getMultiplexerConfig()
   {
      return muxConfig;
   }

   public JChannelFactory getJChannelFactory()
   {
      return delegate;
   }

   public void create() throws Exception
   {
      delegate.create();
   }

   public void destroy() throws Exception
   {
      delegate.destroy();      
   }

   public void start() throws Exception
   {
      delegate.start();
   }

   public void stop() throws Exception
   {
      delegate.stop();
   }

   
}
